import { __decorate } from "tslib";
import { findIconDefinition, icon } from '@fortawesome/fontawesome-svg-core';
import { UUID } from '@genesislcap/foundation-utils';
import { attr, observable } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { foundationIconStyles as styles } from './icon.styles';
import { foundationIconTemplate as template } from './icon.template';
// explicitly declaring for extensibility
export const foundationIconShadowOptions = {
    mode: 'open',
    delegatesFocus: true,
};
export const defaultIconConfig = {};
export var FaPrefixFromStyle;
(function (FaPrefixFromStyle) {
    FaPrefixFromStyle["solid"] = "fas";
    FaPrefixFromStyle["regular"] = "far";
    FaPrefixFromStyle["brand"] = "fab";
})(FaPrefixFromStyle || (FaPrefixFromStyle = {}));
export const faSizeValueDefault = '1x';
export const faSizeValues = [
    '2xs',
    'xs',
    'sm',
    'lg',
    'xl',
    '2xl',
    faSizeValueDefault,
    '2x',
    '3x',
    '4x',
    '5x',
    '6x',
    '7x',
    '8x',
    '9x',
    '10x',
    'stack-1x',
    'stack-2x',
];
/**
 * @tagname %%prefix%%-icon
 */
export class Icon extends FoundationElement {
    constructor() {
        super(...arguments);
        this.isError = false;
        this.size = 'sm';
        this.variant = 'solid';
    }
    nameChanged(oldValue, newValue) {
        newValue ? this.loadIcon() : '';
    }
    sizeChanged(oldValue, newValue) {
        newValue ? this.loadIcon() : '';
    }
    variantChanged(oldValue, newValue) {
        newValue ? this.loadIcon() : '';
    }
    connectedCallback() {
        super.connectedCallback();
        this.loadIcon();
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        this.iElement.replaceChildren();
    }
    getDefaultTooltipConfig() {
        if (!this.name) {
            this.name = 'question-circle';
            this.size = '1x';
            this.variant = 'regular';
        }
    }
    loadIcon() {
        if (!this.iElement) {
            return;
        }
        if (this.tooltip) {
            if (!this.tooltipId) {
                this.tooltipId = this.uuid.createId();
            }
            this.getDefaultTooltipConfig();
        }
        const iconDefinition = findIconDefinition({
            prefix: FaPrefixFromStyle[this.variant],
            iconName: this.name,
        });
        const faIcon = icon(iconDefinition);
        if (faIcon && faIcon.node) {
            this.iElement.innerHTML = '';
            Array.from(faIcon.node).map((n) => this.iElement.appendChild(n));
        }
    }
}
__decorate([
    observable
], Icon.prototype, "iElement", void 0);
__decorate([
    UUID
], Icon.prototype, "uuid", void 0);
__decorate([
    attr({ attribute: 'tooltip-id' })
], Icon.prototype, "tooltipId", void 0);
__decorate([
    attr
], Icon.prototype, "tooltip", void 0);
__decorate([
    attr({ attribute: 'error', mode: 'boolean' })
], Icon.prototype, "isError", void 0);
__decorate([
    attr
], Icon.prototype, "name", void 0);
__decorate([
    attr
], Icon.prototype, "size", void 0);
__decorate([
    attr
], Icon.prototype, "variant", void 0);
/**
 * The Foundation Icon
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-icon\>
 */
export const foundationIcon = Icon.compose(Object.assign({ baseName: 'icon', template,
    styles, shadowOptions: foundationIconShadowOptions }, defaultIconConfig));
