var OptionsDatasource_1;
import { __awaiter, __decorate } from "tslib";
import { ResourceType } from '@genesislcap/foundation-comms';
import { Combobox as FASTComboBox, } from '@microsoft/fast-components';
import { attr, customElement, DOM, observable } from '@microsoft/fast-element';
import { BaseDatasource, getPrefix, logger } from '../utils';
import { dataserverCriteriaBuilder, requestServerCriteriaBuilder } from '../utils';
let OptionsDatasource = OptionsDatasource_1 = class OptionsDatasource extends BaseDatasource {
    constructor() {
        super(...arguments);
        this.emptyDataLabel = 'No matching records';
        this.initalSelectedValueInitialized = false;
        this.isSnapshot = true;
        this.resizeObserverCallback = (entries) => {
            entries.forEach((resizeObserverEntry) => {
                this.checkOverflow(resizeObserverEntry.target);
            });
        };
        this.selectChangeHandler = (e) => {
            var _a;
            const selectedOption = (_a = this.options) === null || _a === void 0 ? void 0 : _a.find((option) => String(option.label) === e.target.value);
            this.select.$emit('value-change', selectedOption);
        };
    }
    initialSelectedValueChanged(oldValue, newValue) {
        if (oldValue && newValue) {
            this.initalSelectedValueInitialized = false;
            this.clearData();
            this.fetchData();
        }
    }
    connectedCallback() {
        const _super = Object.create(null, {
            connectedCallback: { get: () => super.connectedCallback }
        });
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            _super.connectedCallback.call(this);
            this.fields = this.labelField ? (_a = this.valueField) === null || _a === void 0 ? void 0 : _a.concat(' ', this.labelField) : this.valueField;
            this.select = this.parentElement;
            this.optionElement = (_b = this.optionElement) !== null && _b !== void 0 ? _b : `${getPrefix(this.select)}-option`;
            this.select.addEventListener('change', this.selectChangeHandler);
            this.resizeObserver = new ResizeObserver(this.resizeObserverCallback);
            this.fetchData();
            if (!this.valueField) {
                logger.warn('Specify field for values in options (e.g value-field="VALUE") and optionally field for label (e.g label-field="NAME")');
            }
        });
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        this.resizeObserver.disconnect();
        this.select.removeEventListener('change', this.selectChangeHandler);
    }
    fetchData() {
        if (this.select &&
            this.valueField &&
            ((this.resourceName && this.connect.isConnected) || this.data)) {
            if (this.select instanceof FASTComboBox && this.select.async && this.initialSelectedValue) {
                this.asyncModeIntialization();
            }
            else {
                this.fetchGenesisData();
            }
        }
    }
    asyncModeIntialization() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.datasource.init(this['datasourceOptions'](), true);
            const resourceType = this.datasource.resourceType;
            if (resourceType === ResourceType.DATASERVER) {
                dataserverCriteriaBuilder(this, this.initialSelectedValue);
            }
            if (resourceType === ResourceType.REQUEST_SERVER) {
                requestServerCriteriaBuilder(this, this.initialSelectedValue);
            }
        });
    }
    clearData() {
        Array.from(this.select.children).forEach((x) => {
            if (!(x instanceof OptionsDatasource_1)) {
                this.select.removeChild(x);
            }
        });
        this.select.value = '';
        this.select.initialValue = '';
    }
    checkOverflow(element) {
        const content = element.shadowRoot.querySelector('.content');
        if (content.scrollWidth > content.clientWidth) {
            element.setAttribute('title', element.textContent);
        }
        else {
            element.removeAttribute('title');
        }
    }
    syncComponentData(snapshotData) {
        var _a;
        const rowData = snapshotData !== null && snapshotData !== void 0 ? snapshotData : Array.from(this.rowsSyncedWithStream.values());
        if (rowData.length > 0) {
            this.options = rowData.map((row) => ({
                value: row[this.valueField],
                label: this.labelFormatter
                    ? this.labelFormatter(row[this.labelField])
                    : row[this.labelField],
            }));
            const listboxOptions = this.options.map(({ value, label }) => {
                const option = document.createElement(this.optionElement);
                option.textContent = label ? label : value;
                option.value = value;
                return option;
            });
            listboxOptions.forEach((x) => {
                this.select.appendChild(x);
                this.resizeObserver.observe(x);
            });
            if (!this.initalSelectedValueInitialized ||
                (this.select instanceof FASTComboBox && !this.select.async)) {
                this.resetCombobox();
                const initialLabel = this.initialSelectedValue || this.initialSelectedValue === 0
                    ? (_a = this.options.find((x) => x.value === this.initialSelectedValue)) === null || _a === void 0 ? void 0 : _a.label
                    : '';
                DOM.queueUpdate(() => {
                    this.select.value = String(initialLabel);
                });
                this.initalSelectedValueInitialized = true;
            }
        }
        else {
            const option = document.createElement(this.optionElement);
            option.textContent = this.emptyDataLabel;
            option.value = '';
            option.disabled = true;
            this.select.appendChild(option);
        }
    }
    resetCombobox() {
        if (this.select && this.select instanceof FASTComboBox) {
            this.select.resetCombobox();
        }
    }
    handleStreamInserts(insertedRows) {
        logger.debug('handleStreamInserts - Method not implemented.');
    }
    handleStreamDeletes(deletedRows) {
        logger.debug('handleStreamDeletes - Method not implemented.');
    }
    handleStreamUpdates(updatedRows) {
        logger.debug('handleStreamUpdates - Method not implemented.');
    }
};
__decorate([
    attr({ attribute: 'label-field' })
], OptionsDatasource.prototype, "labelField", void 0);
__decorate([
    attr({ attribute: 'option-element' })
], OptionsDatasource.prototype, "optionElement", void 0);
__decorate([
    attr({ attribute: 'value-field' })
], OptionsDatasource.prototype, "valueField", void 0);
__decorate([
    attr({ attribute: 'empty-data-label' })
], OptionsDatasource.prototype, "emptyDataLabel", void 0);
__decorate([
    observable
], OptionsDatasource.prototype, "labelFormatter", void 0);
__decorate([
    observable
], OptionsDatasource.prototype, "initialSelectedValue", void 0);
OptionsDatasource = OptionsDatasource_1 = __decorate([
    customElement({
        name: 'options-datasource',
    })
], OptionsDatasource);
export { OptionsDatasource };
